var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React from 'react';
import { forwardRef } from '../../utils/reactPolyfill';
import hoistStatics from 'hoist-non-react-statics';
import { isFunction } from '../../utils/fp';

var isStatelessComponent = function isStatelessComponent(component) {
	return !component || isFunction(component) && !('render' in component.prototype);
};

var getDisplayName = function getDisplayName(component) {
	return component.displayName || component.name;
};

export default function hoist(WrappedComponent) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
	var displayName = options.displayName;

	return function createHoistedComponent(TargetComponent) {
		var HoistedComponent = TargetComponent;
		if (!isStatelessComponent(TargetComponent) && !isStatelessComponent(WrappedComponent)) {
			HoistedComponent = forwardRef(function (props, ref) {
				return React.createElement(TargetComponent, _extends({}, props, { forwardedRef: ref }));
			});
			HoistedComponent.displayName = getDisplayName(TargetComponent);
			HoistedComponent.defaultProps = TargetComponent.defaultProps;
		}

		if (WrappedComponent) {
			HoistedComponent.WrappedComponent = WrappedComponent;
			hoistStatics(HoistedComponent, WrappedComponent, { render: true });
		}
		if (displayName) {
			HoistedComponent.displayName = displayName;
		} else {
			var originalDisplayName = getDisplayName(WrappedComponent);
			var prefix = getDisplayName(HoistedComponent);
			HoistedComponent.displayName = prefix + '(' + originalDisplayName + ')';
		}
		return HoistedComponent;
	};
}

var extractRef = function extractRef(props) {
	var forwardedRef = props.forwardedRef,
	    other = _objectWithoutProperties(props, ['forwardedRef']);

	return forwardedRef ? _extends({}, other, { ref: forwardedRef }) : other;
};
export { extractRef };