var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React, { Children } from 'react';
import PropTypes from '../../utils/PropTypes';
import { flatMap } from '../../utils/fp';
import FrameView from '../../containers/FrameView';
import IndexView from '../../containers/IndexView';
import LoginView from '../../containers/LoginView';
import DataTableView from '../../containers/DataTableView';
import NotFoundView from '../../containers/NotFoundView';
import Route from '../../components/EnhancedRoute';
import { CreaterFormDetailView, UpdaterFormDetailView } from '../../containers/FormDetailViews';

export default function NavigatorSchema() {
	return React.createElement('noscript', null);
}

NavigatorSchema.propTypes = {
	basename: PropTypes.string,
	noBreadcrumb: PropTypes.bool,
	noHomeBreadcrumb: PropTypes.bool,
	noUserMenu: PropTypes.bool,
	index: PropTypes.component,
	login: PropTypes.component,
	frame: PropTypes.component,
	tableComponent: PropTypes.component,
	updaterComponent: PropTypes.component,
	createrComponent: PropTypes.component,
	notFound: PropTypes.component
};

NavigatorSchema.defaultProps = {
	basename: '/',
	noBreadcrumb: false,
	noHomeBreadcrumb: false,
	noUserMenu: false,
	index: IndexView,
	login: LoginView,
	frame: FrameView,
	tableComponent: DataTableView,
	updaterComponent: UpdaterFormDetailView,
	createrComponent: CreaterFormDetailView,
	notFound: NotFoundView
};

NavigatorSchema.configuration = {
	name: 'navigator',
	propType: PropTypes.object,
	pipe: function pipe(_ref) {
		var children = _ref.children,
		    other = _objectWithoutProperties(_ref, ['children']);

		Object.assign(this, other);

		var tableComponent = this.tableComponent,
		    updaterComponent = this.updaterComponent,
		    createrComponent = this.createrComponent,
		    welcome = this.index,
		    notFound = this.notFound,
		    noBreadcrumb = this.noBreadcrumb;


		var sidebarChildren = [];
		var topChildren = [];

		Children.toArray(children).forEach(function (child) {
			if (child.props.top) {
				topChildren.push(child);
			} else {
				sidebarChildren.push(child);
			}
		});

		var getMenu = function getMenu(children) {
			var next = function next(child) {
				var keyPaths = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
				var rootPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '/';

				var _child$props = child.props,
				    children = _child$props.children,
				    props = _objectWithoutProperties(_child$props, ['children']);

				var nextRootPath = rootPath;

				props.isInternalPath = !/^https?:\/\//i.test(props.path);

				if (props.path) {
					if (props.isInternalPath) {
						props.path = (rootPath + props.path).replace(/\/\//, '/');
						nextRootPath = props.path;

						var menuTitle = props.title,
						    pageTitle = props.pageTitle,
						    breadcrumbTitle = props.breadcrumbTitle;

						props.pageTitle = pageTitle || menuTitle;
						props.breadcrumbTitle = breadcrumbTitle || props.pageTitle;
					}

					props.menuKey = props.path;
				} else {
					props.menuKey = '@' + keyPaths.join('-');
				}

				if (children) {
					props.children = Children.map(children, function (child, index) {
						return next(child, keyPaths.concat(index), nextRootPath);
					});
				}

				return props;
			};

			return next({ props: { children: children } }).children;
		};

		var getRoutes = function getRoutes(menus) {
			if (!menus || !menus.length) return [];

			return flatMap(menus, function (_ref2, index) {
				var children = _ref2.children,
				    route = _objectWithoutProperties(_ref2, ['children']);

				var props = _extends({ key: index }, route);

				if (children) {
					return getRoutes(children);
				} else {
					var _routes = [];
					var table = props.table,
					    path = props.path,
					    breadcrumbTitle = props.breadcrumbTitle;


					if (!props.component && !props.render) {
						props.component = table ? tableComponent : notFound;
					}

					if (table) {
						var breadcrumbParent = { path: path, breadcrumbTitle: breadcrumbTitle };

						var updaterParams = props.updaterParams,
						    createrParams = props.createrParams,
						    pageTitle = props.pageTitle,
						    restProps = _objectWithoutProperties(props, ['updaterParams', 'createrParams', 'pageTitle']);

						_routes.push(React.createElement(Route, _extends({}, restProps, {
							component: updaterComponent
						}, updaterParams, {
							breadcrumbParent: breadcrumbParent,
							path: path + '/update/:key'
						})), React.createElement(Route, _extends({}, restProps, {
							component: createrComponent
						}, createrParams, {
							breadcrumbParent: breadcrumbParent,
							path: path + '/create/'
						})));
					}

					_routes.push(React.createElement(Route, _extends({}, props, { exact: true })));
					return _routes;
				}
			});
		};

		var sidebarMenu = getMenu(sidebarChildren);
		var topMenu = getMenu(topChildren);
		var userRoutes = [].concat(_toConsumableArray(getRoutes(sidebarMenu)), _toConsumableArray(getRoutes(topMenu)));
		var routes = [React.createElement(Route, { key: '_index', exact: true, path: '/', menuKey: '/', component: welcome })].concat(_toConsumableArray(userRoutes), [React.createElement(Route, { key: '_notFound', component: notFound })]);

		this.menus = sidebarMenu;
		this.topMenu = topMenu;
		this.routes = routes;
		return this;
	}
};