var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/* eslint-disable react/display-name */

import React, { Children } from 'react';
import PropTypes from '../../utils/PropTypes';
import { returnsArgument } from 'empty-functions';
import parseAPIPath from '../../utils/parseAPIPath';
import { isFunction, isObject } from '../../utils/fp';
import FieldGateway from '../../components/FieldGateway';
import createRendererState from './createRendererState';

export default function TableSchema() {
	return React.createElement('noscript', null);
}

TableSchema.propTypes = {
	name: PropTypes.string.isRequired,
	api: PropTypes.stringOrObject,
	mapOnFetchResponse: PropTypes.func,
	mapOnFetchOneResponse: PropTypes.func,
	mapOnSave: PropTypes.func,
	uniqueKey: PropTypes.string,
	maxSelections: PropTypes.number,
	extend: PropTypes.object,
	viewSize: PropTypes.string,
	useCursor: PropTypes.bool
};

TableSchema.defaultProps = {
	mapOnFetchResponse: returnsArgument,
	mapOnFetchOneResponse: returnsArgument,
	mapOnSave: returnsArgument,
	uniqueKey: '_id',
	maxSelections: -1,
	extend: {},
	viewSize: 'middle',
	useCursor: false
};

TableSchema.configuration = {
	name: 'tables',
	propType: PropTypes.object,
	pipe: function pipe(_ref) {
		var name = _ref.name,
		    api = _ref.api,
		    children = _ref.children,
		    other = _objectWithoutProperties(_ref, ['name', 'api', 'children']);

		children = Children.toArray(children);
		var firstChild = children[0];
		if (firstChild && children.length === 1 && firstChild.type === 'noscript') {
			children = Children.toArray(firstChild.props.children);
		}

		var uniqueKey = void 0;

		var renderers = [];

		children.forEach(function (child, index) {
			var _child$props = child.props,
			    inForm = _child$props.inForm,
			    inQuery = _child$props.inQuery,
			    inTable = _child$props.inTable,
			    unique = _child$props.unique,
			    rendererProp = _child$props.renderer,
			    props = _objectWithoutProperties(_child$props, ['inForm', 'inQuery', 'inTable', 'unique', 'renderer']);

			var name = props.name;


			if (!uniqueKey && unique) uniqueKey = name;

			var key = child.key || index;
			var Component = child.type;
			var options = { key: key, Component: Component, name: name };

			var makeRender = function makeRender(renderProp) {
				return function (store, otherProps) {
					var state = createRendererState(store, options);

					if (isFunction(otherProps)) {
						var _children = otherProps;
						otherProps = { children: _children };
					}

					var createRenderFn = function createRenderFn(inIssuer) {
						return function (extraProps) {
							var instanceProps = _extends({}, props, extraProps);

							if (isFunction(inIssuer)) {
								return inIssuer(instanceProps, state);
							}

							if (isObject(inIssuer)) {
								return React.createElement(Component, _extends({}, instanceProps, inIssuer));
							}

							if (isFunction(Component[renderProp])) {
								return Component[renderProp](instanceProps, state);
							}

							if (renderProp === 'renderTable') return state.value;
							return React.createElement(Component, instanceProps);
						};
					};

					var renderer = function renderer(ctx) {
						if (inTable) ctx.when(ctx.is.TABLE, createRenderFn(inTable));
						if (inForm) {
							ctx.when(ctx.is.UPDATER || ctx.is.CREATER, createRenderFn(inForm));
						}
						if (inQuery) ctx.when(ctx.is.QUERIER, createRenderFn(inQuery));
						if (rendererProp) rendererProp(ctx);
					};

					return React.createElement(FieldGateway, _extends({
						state: state,
						props: props,
						renderer: renderer
					}, otherProps));
				};
			};
			renderers.push({
				props: props,
				renderTable: makeRender('renderTable'),
				renderQuery: makeRender('renderQuery'),
				renderForm: makeRender('renderForm')
			});
		});

		var table = _extends({}, other, {
			api: parseAPIPath(api || name),
			renderers: renderers
		});

		if (uniqueKey) table.uniqueKey = uniqueKey;

		this[name] = table;

		if (renderers.length && !table.uniqueKey) {
			console.error('Table "' + name + '" is missing uniqueKey!');
		}

		return this;
	},

	initialData: {}
};