'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = NavigatorSchema;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _PropTypes = require('../../utils/PropTypes');

var _PropTypes2 = _interopRequireDefault(_PropTypes);

var _fp = require('../../utils/fp');

var _FrameView = require('../../containers/FrameView');

var _FrameView2 = _interopRequireDefault(_FrameView);

var _IndexView = require('../../containers/IndexView');

var _IndexView2 = _interopRequireDefault(_IndexView);

var _LoginView = require('../../containers/LoginView');

var _LoginView2 = _interopRequireDefault(_LoginView);

var _DataTableView = require('../../containers/DataTableView');

var _DataTableView2 = _interopRequireDefault(_DataTableView);

var _NotFoundView = require('../../containers/NotFoundView');

var _NotFoundView2 = _interopRequireDefault(_NotFoundView);

var _EnhancedRoute = require('../../components/EnhancedRoute');

var _EnhancedRoute2 = _interopRequireDefault(_EnhancedRoute);

var _FormDetailViews = require('../../containers/FormDetailViews');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function NavigatorSchema() {
	return _react2.default.createElement('noscript', null);
}

NavigatorSchema.propTypes = {
	basename: _PropTypes2.default.string,
	noBreadcrumb: _PropTypes2.default.bool,
	noHomeBreadcrumb: _PropTypes2.default.bool,
	noUserMenu: _PropTypes2.default.bool,
	index: _PropTypes2.default.component,
	login: _PropTypes2.default.component,
	frame: _PropTypes2.default.component,
	tableComponent: _PropTypes2.default.component,
	updaterComponent: _PropTypes2.default.component,
	createrComponent: _PropTypes2.default.component,
	notFound: _PropTypes2.default.component
};

NavigatorSchema.defaultProps = {
	basename: '/',
	noBreadcrumb: false,
	noHomeBreadcrumb: false,
	noUserMenu: false,
	index: _IndexView2.default,
	login: _LoginView2.default,
	frame: _FrameView2.default,
	tableComponent: _DataTableView2.default,
	updaterComponent: _FormDetailViews.UpdaterFormDetailView,
	createrComponent: _FormDetailViews.CreaterFormDetailView,
	notFound: _NotFoundView2.default
};

NavigatorSchema.configuration = {
	name: 'navigator',
	propType: _PropTypes2.default.object,
	pipe: function pipe(_ref) {
		var children = _ref.children,
		    other = _objectWithoutProperties(_ref, ['children']);

		Object.assign(this, other);

		var tableComponent = this.tableComponent,
		    updaterComponent = this.updaterComponent,
		    createrComponent = this.createrComponent,
		    welcome = this.index,
		    notFound = this.notFound,
		    noBreadcrumb = this.noBreadcrumb;


		var sidebarChildren = [];
		var topChildren = [];

		_react.Children.toArray(children).forEach(function (child) {
			if (child.props.top) {
				topChildren.push(child);
			} else {
				sidebarChildren.push(child);
			}
		});

		var getMenu = function getMenu(children) {
			var next = function next(child) {
				var keyPaths = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
				var rootPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '/';

				var _child$props = child.props,
				    children = _child$props.children,
				    props = _objectWithoutProperties(_child$props, ['children']);

				var nextRootPath = rootPath;

				props.isInternalPath = !/^https?:\/\//i.test(props.path);

				if (props.path) {
					if (props.isInternalPath) {
						props.path = (rootPath + props.path).replace(/\/\//, '/');
						nextRootPath = props.path;

						var menuTitle = props.title,
						    pageTitle = props.pageTitle,
						    breadcrumbTitle = props.breadcrumbTitle;

						props.pageTitle = pageTitle || menuTitle;
						props.breadcrumbTitle = breadcrumbTitle || props.pageTitle;
					}

					props.menuKey = props.path;
				} else {
					props.menuKey = '@' + keyPaths.join('-');
				}

				if (children) {
					props.children = _react.Children.map(children, function (child, index) {
						return next(child, keyPaths.concat(index), nextRootPath);
					});
				}

				return props;
			};

			return next({ props: { children: children } }).children;
		};

		var getRoutes = function getRoutes(menus) {
			if (!menus || !menus.length) return [];

			return (0, _fp.flatMap)(menus, function (_ref2, index) {
				var children = _ref2.children,
				    route = _objectWithoutProperties(_ref2, ['children']);

				var props = _extends({ key: index }, route);

				if (children) {
					return getRoutes(children);
				} else {
					var _routes = [];
					var table = props.table,
					    path = props.path,
					    breadcrumbTitle = props.breadcrumbTitle;


					if (!props.component && !props.render) {
						props.component = table ? tableComponent : notFound;
					}

					if (table) {
						var breadcrumbParent = { path: path, breadcrumbTitle: breadcrumbTitle };

						var updaterParams = props.updaterParams,
						    createrParams = props.createrParams,
						    pageTitle = props.pageTitle,
						    restProps = _objectWithoutProperties(props, ['updaterParams', 'createrParams', 'pageTitle']);

						_routes.push(_react2.default.createElement(_EnhancedRoute2.default, _extends({}, restProps, {
							component: updaterComponent
						}, updaterParams, {
							breadcrumbParent: breadcrumbParent,
							path: path + '/update/:key'
						})), _react2.default.createElement(_EnhancedRoute2.default, _extends({}, restProps, {
							component: createrComponent
						}, createrParams, {
							breadcrumbParent: breadcrumbParent,
							path: path + '/create/'
						})));
					}

					_routes.push(_react2.default.createElement(_EnhancedRoute2.default, _extends({}, props, { exact: true })));
					return _routes;
				}
			});
		};

		var sidebarMenu = getMenu(sidebarChildren);
		var topMenu = getMenu(topChildren);
		var userRoutes = [].concat(_toConsumableArray(getRoutes(sidebarMenu)), _toConsumableArray(getRoutes(topMenu)));
		var routes = [_react2.default.createElement(_EnhancedRoute2.default, { key: '_index', exact: true, path: '/', menuKey: '/', component: welcome })].concat(_toConsumableArray(userRoutes), [_react2.default.createElement(_EnhancedRoute2.default, { key: '_notFound', component: notFound })]);

		this.menus = sidebarMenu;
		this.topMenu = topMenu;
		this.routes = routes;
		return this;
	}
};