'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = TableSchema;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _PropTypes = require('../../utils/PropTypes');

var _PropTypes2 = _interopRequireDefault(_PropTypes);

var _emptyFunctions = require('empty-functions');

var _parseAPIPath = require('../../utils/parseAPIPath');

var _parseAPIPath2 = _interopRequireDefault(_parseAPIPath);

var _fp = require('../../utils/fp');

var _FieldGateway = require('../../components/FieldGateway');

var _FieldGateway2 = _interopRequireDefault(_FieldGateway);

var _createRendererState = require('./createRendererState');

var _createRendererState2 = _interopRequireDefault(_createRendererState);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; } /* eslint-disable react/display-name */

function TableSchema() {
	return _react2.default.createElement('noscript', null);
}

TableSchema.propTypes = {
	name: _PropTypes2.default.string.isRequired,
	api: _PropTypes2.default.stringOrObject,
	mapOnFetchResponse: _PropTypes2.default.func,
	mapOnFetchOneResponse: _PropTypes2.default.func,
	mapOnSave: _PropTypes2.default.func,
	uniqueKey: _PropTypes2.default.string,
	maxSelections: _PropTypes2.default.number,
	extend: _PropTypes2.default.object,
	viewSize: _PropTypes2.default.string,
	useCursor: _PropTypes2.default.bool
};

TableSchema.defaultProps = {
	mapOnFetchResponse: _emptyFunctions.returnsArgument,
	mapOnFetchOneResponse: _emptyFunctions.returnsArgument,
	mapOnSave: _emptyFunctions.returnsArgument,
	uniqueKey: '_id',
	maxSelections: -1,
	extend: {},
	viewSize: 'middle',
	useCursor: false
};

TableSchema.configuration = {
	name: 'tables',
	propType: _PropTypes2.default.object,
	pipe: function pipe(_ref) {
		var name = _ref.name,
		    api = _ref.api,
		    children = _ref.children,
		    other = _objectWithoutProperties(_ref, ['name', 'api', 'children']);

		children = _react.Children.toArray(children);
		var firstChild = children[0];
		if (firstChild && children.length === 1 && firstChild.type === 'noscript') {
			children = _react.Children.toArray(firstChild.props.children);
		}

		var uniqueKey = void 0;

		var renderers = [];

		children.forEach(function (child, index) {
			var _child$props = child.props,
			    inForm = _child$props.inForm,
			    inQuery = _child$props.inQuery,
			    inTable = _child$props.inTable,
			    unique = _child$props.unique,
			    rendererProp = _child$props.renderer,
			    props = _objectWithoutProperties(_child$props, ['inForm', 'inQuery', 'inTable', 'unique', 'renderer']);

			var name = props.name;


			if (!uniqueKey && unique) uniqueKey = name;

			var key = child.key || index;
			var Component = child.type;
			var options = { key: key, Component: Component, name: name };

			var makeRender = function makeRender(renderProp) {
				return function (store, otherProps) {
					var state = (0, _createRendererState2.default)(store, options);

					if ((0, _fp.isFunction)(otherProps)) {
						var _children = otherProps;
						otherProps = { children: _children };
					}

					var createRenderFn = function createRenderFn(inIssuer) {
						return function (extraProps) {
							var instanceProps = _extends({}, props, extraProps);

							if ((0, _fp.isFunction)(inIssuer)) {
								return inIssuer(instanceProps, state);
							}

							if ((0, _fp.isObject)(inIssuer)) {
								return _react2.default.createElement(Component, _extends({}, instanceProps, inIssuer));
							}

							if ((0, _fp.isFunction)(Component[renderProp])) {
								return Component[renderProp](instanceProps, state);
							}

							if (renderProp === 'renderTable') return state.value;
							return _react2.default.createElement(Component, instanceProps);
						};
					};

					var renderer = function renderer(ctx) {
						if (inTable) ctx.when(ctx.is.TABLE, createRenderFn(inTable));
						if (inForm) {
							ctx.when(ctx.is.UPDATER || ctx.is.CREATER, createRenderFn(inForm));
						}
						if (inQuery) ctx.when(ctx.is.QUERIER, createRenderFn(inQuery));
						if (rendererProp) rendererProp(ctx);
					};

					return _react2.default.createElement(_FieldGateway2.default, _extends({
						state: state,
						props: props,
						renderer: renderer
					}, otherProps));
				};
			};
			renderers.push({
				props: props,
				renderTable: makeRender('renderTable'),
				renderQuery: makeRender('renderQuery'),
				renderForm: makeRender('renderForm')
			});
		});

		var table = _extends({}, other, {
			api: (0, _parseAPIPath2.default)(api || name),
			renderers: renderers
		});

		if (uniqueKey) table.uniqueKey = uniqueKey;

		this[name] = table;

		if (renderers.length && !table.uniqueKey) {
			console.error('Table "' + name + '" is missing uniqueKey!');
		}

		return this;
	},

	initialData: {}
};